
#' Generate and encapsulate initial values
#'
#' @param States Length-4 vector representing the four methylation states
#' @param isp Length-4 vector representing the initial state distribution 
#' @param ie Length-4 vector, the parameter in the transition model, the logarithm of the characteristic lengths.
#' @param imuBH the mean of the underlying methylation proportion for the 'Both High' loci
#' @param imuBL the mean of the underlying methylation proportion for the 'Both Low' loci
#' @param icBHX variance of \eqn{pX_{ij}} in the 'Both High' regions
#' @param icBLX variance of \eqn{pX_{ij}} in the 'Both Low' regions
#' @param icBHY variance of \eqn{pY_{ij}} in the 'Both High' regions
#' @param icBLY variance of \eqn{py_{ij}} in the 'Both Low' regions
#' @param D tunning parameter in model, 
#' which represent the minimun difference between the mean of \eqn{pX_{ij}} and \eqn{pY_{ij}} in the DMR regions
#' @param imuHyper1 the mean of \eqn{logit(pX_{ij})} for the 'HyperM' loci
#' @param imuHypo1 the mean of \eqn{logit(pX_{ij})} for  the 'HypoM' loci
#' @param s113 variance of \eqn{logit(pX_{ij})} in the 'HyperM' regions
#' @param s223 variance of \eqn{logit(pY_{ij})} in the 'HyperM' regions
#' @param rho3 Correlation of \eqn{logit(pX_{ij})} and \eqn{logit(pY_{ij})} for the 'HyperM' loci
#' @param s114 variance of \eqn{logit(pX_{ij})} in the 'HypoM' regions
#' @param s224 variance of \eqn{logit(pY_{ij})} in the 'HypoM' regions
#' @param rho4 Correlation of \eqn{logit(pX_{ij})} and \eqn{logit(pY_{ij})} for the 'HypoM' loci
#' @param alpha  tunning parameter in model, 
#' which used to truncating the covariant matrices of bivariate normals of DMC's responses. 
#' 
#' @return Simply a list with all information are the same with input, 
#' if the initial values of all parameters are entered, 
#' otherwise the initial values are generated randomly.


#' @export
#'
#' @examples
InitialValues <- function(States = c("Both High", "Both Low", "HyperM", "HypoM"),
                          isp=NULL,ie=NULL,imuBH=NULL,imuBL=NULL,
                          icBHX=NULL,icBLX=NULL,icBHY=NULL,icBLY=NULL, D=0.3, 
                          imuHyper1 = NULL, imuHypo1 = NULL,
                          s113 = NULL,  s223 = NULL, rho3 = NULL,
                          s114 = NULL, s224 = NULL, rho4 = NULL,alpha = 0.8){
  if (is.null(isp)) {
    #isp <- sample(1:100,4,replace=TRUE)
    #isp <- isp/sum(isp)
    isp <- rdirichlet(1,rep(1,4))    #cyy
  }
  if (is.null(ie)) {
    ie <- sample(seq(1,20,0.1),4)
  }
  if (is.null(imuBH)) {imuBH <- runif(1,0.5,1)}
  if (is.null(imuBL)) {imuBL <- runif(1,0,0.5)}
  if (is.null(icBHX)) {icBHX <- runif(1,0.01,0.1)}
  if (is.null(icBLX)) {icBLX <- runif(1,0.01,0.1)}
  if (is.null(icBHY)) {icBHY <- runif(1,0.001,0.005)}
  if (is.null(icBLY)) {icBLY <- runif(1,0.001,0.005)}
  if (is.null(imuHyper1)) { imuHyper1 <- runif(1,logit(0.1),logit(0.4))}
  if (is.null(imuHypo1)) {imuHypo1 <- runif(1,logit(0.4),logit(0.8))}
  if (is.null(s113)) {s113 <- runif(1,0.1,1)}
  if (is.null(s223)) {s223 <- runif(1,0.1,1)}
  if (is.null(rho3)) {rho3 <- runif(1,0,0.8)}
  if (is.null(s114)) {s114 <- runif(1,0.1,1)}
  if (is.null(s224)) {s224 <- runif(1,0.1,1)}
  if (is.null(rho4)) {rho4 <- runif(1,0,0.8)}
  
  isigmaHyper <- matrix(c(s113,rho3*sqrt(s113*s223),rho3*sqrt(s113*s223),s223),2)
  bound3 <- Bound_mu2('HyperM',c(imuHyper1,0), isigmaHyper,alpha,D)
  # imuHyper2 <- runif(1,bound3,15)
  imuHyper2 <- rtruncnorm(1,bound3,15,ilogit(imuHyper1+0.3),0.01)
  
  
  isigmaHypo <- matrix(c(s113,rho4*sqrt(s113*s223),rho4*sqrt(s113*s223),s223),2)
  bound4 <- Bound_mu2('HypoM',c(imuHypo1,0), isigmaHypo, alpha,D)
  # imuHypo2 <- runif(1,-15,bound4)
  imuHypo2 <- rtruncnorm(1,-15,bound4,ilogit(imuHypo1-0.3),0.01)
  
  
  initials = list(States=States, StartProb=isp, e = ie, 
             muBH=imuBH, muBL=imuBL, 
             muHyper=c(imuHyper1,imuHyper2), muHypo=c(imuHypo1,imuHypo2),
             cBHX=icBHX, cBLX=icBLX, 
             cBHY=icBHY, cBLY=icBLY,
             sigmaHyper=isigmaHyper, sigmaHypo=isigmaHypo) 
  return(initials)

}

