#' Read WGBS data of tumour cell and normal cell loaded from GEO,
#' then reform them into a 2-by-7 data frame. 
#'
#' @param TumourFile The name of the file containing the WGBS data 
#' of tumour cell to be read from.
#' @param NormalFile The name of the file containing the WGBS data 
#' of normal cell to be read from.
#'
#' @return A data frame with two rows and 7 columns. 
#' Rows represent tumour cell and normal cell in order.
#' And Columns represent chromosome 'chr', 
#' starting positions of CpG sites 'starting',
#' the ending positions of CpG sites 'ending', 
#' the numbers of methylated 
#' cytosine reads and read depths from tumour cell \eqn{X,nX}, 
#' the numbers of methylated 
#' cytosine reads and read depths from normal cell \eqn{Y,nY}
#' @export
#'
#' @examples
MergeData <- function(TumourFile,NormalFile){
  Tumour <- read.table(TumourFile,header = FALSE, 
                      sep="\t",stringsAsFactors=FALSE, quote="")
  Normal <- read.table(NormalFile,header = FALSE,
                      sep="\t",stringsAsFactors=FALSE, quote="")
  colnames(Normal) <- c("chr","start","end","m/u","bbb","ccc")
  colnames(Tumour) <- c("chr","start","end","m/u","bbb","ccc")
  
  ALL <- inner_join(Normal[,-(5:6)],Tumour[,-(5:6)],by=c("chr","start","end"))
  ALL <- ALL[order(ALL$chr,ALL$start),]
  N <- nrow(ALL)
  Xobs <- t(sapply(str_sub(ALL[1:N,4],2,-2),
                  function(x){as.numeric(unlist(strsplit(x,"/")))}))
  Yobs <- t(sapply(str_sub(ALL[1:N,5],2,-2),
                  function(x){as.numeric(unlist(strsplit(x,"/")))}))
  data <- cbind(ALL[1:N,1:3],Xobs,Yobs)
  colnames(data) <- c("chr","start","end","X","nX","Y","nY")
  L <- diff(data$start)
  data = list(S = rep(0,N), pX = rep(0,N), pY = rep(0,N), L = L, site=data$start,
              nX = matrix(data$nX,N), nY = matrix(data$nY,N),
              X = matrix(data$X,N), Y = matrix(data$Y,N) )
  return(as.data.frame(data))
}




