#' Given \eqn{\Theta^{(t)}} and the Updated state, 
#' sample the underlying probability pX and pY.
#'
#' @param HMM list of the informations of the HMM 
#' including states, startprob and the mean and variance 
#' at each states. 
#' @param data Integrated data, including all the information from WGBS data used in the estimation . 
#' @param states.i Length-N vector representing the hidden states.
#' @param pX.i Length-N vector representing the underlying prob of X.
#' @param pY.i Length-N vector representing the underlying prob of Y.
#' @param adspXi Sd of the proposal density of pX
#' @param adspYi Sd of the proposal density of pY
#'
#' @return A list containing
#' \item{pX.i}{Updated pX.i}
#' \item{pY.i}{Updated pY.i}
#' \item{index.i1}{the index of the CpG sites in each state.}
#' \item{COUNTS}{the number of CpG sites in each state.}
#' @export
#'
#' @examples   
SampleUnderProb <- function(HMM,data,states.i,
                            pX.i,pY.i,adspXi,adspYi){
  N <- nrow(data$X)
  J <- ncol(data$Y)
  index.i1 <- sapply(HMM$States,
                    function(x){which(states.i==x)},simplify=FALSE)
  COUNTS <- sapply(index.i1,length) 
  for(i in 1:4){
    if(is.na(COUNTS[i])){COUNTS[i]=''}
  }
  pX.s <- matrix(rtn(N*J,0,1,pX.i,adspXi),N)
  pY.s <- matrix(rtn(N*J,0,1,pY.i,adspYi),N)
  
  f <- function(pX,pY){
    C <- dbinom(data$X,data$nX,pX,log=TRUE) + 
      dbinom(data$Y,data$nY,pY,log=TRUE)
    p <- rep(-9999,N)
    if(COUNTS[1] >= 1){
      # iii <- index.i1[[1]]
      # p[iii] <- rowSums(as.matrix(C[iii,] + 
      #                               log(dtruncnorm(pX[iii],0,1,HMM$muBH,sqrt(HMM$cBHX))) +
      #                               log(dtruncnorm(pY[iii],0,1,pX[iii],sqrt(HMM$cBHY)))))
      iii = index.i1[[1]]
      C11 = log( pnorm(1,pX[iii],sqrt(HMM$cBHY))-pnorm(0,pX[iii],sqrt(HMM$cBHY)) )
      p[iii] = rowSums(as.matrix(C[iii]+dnorm(pX[iii],HMM$muBH,sqrt(HMM$cBHX),log=TRUE)+
                                   dnorm(pY[iii],pX[iii],sqrt(HMM$cBHY),log=TRUE)-C11))
      }
    if(COUNTS[2] >=1){
      # iii <- index.i1[[2]]
      # p[iii] <- rowSums(as.matrix(C[iii,] + 
      #                               log(dtruncnorm(pX[iii],0,1,HMM$muBL,sqrt(HMM$cBLX))) +
      #                               log(dtruncnorm(pY[iii],0,1,pX[iii],sqrt(HMM$cBLY)))))
      iii = index.i1[[2]]
      C22 = log( pnorm(1,pX[iii],sqrt(HMM$cBLY))-pnorm(0,pX[iii],sqrt(HMM$cBLY)) )
      p[iii] = rowSums(as.matrix(C[iii]+dnorm(pX[iii],HMM$muBL,sqrt(HMM$cBLX),log=TRUE)+
                                   dnorm(pY[iii],pX[iii],sqrt(HMM$cBLY),log=TRUE)-C22))
      }
    if(COUNTS[3] >= 1){
      iii <- index.i1[[3]]
      lpX <- logit(pX[iii]); lpY <- logit(pY[iii])
      logJ_34 <- -(log(pX[iii])+log(pY[iii])+log(1-pX[iii])+log(1-pY[iii]))
      p[iii] <- rowSums(as.matrix(C[iii,] + 
                                    matrix(dmvnorm(cbind(c(lpX),c(lpY)),
                                                   mean=HMM$muHyper,sigma=HMM$sigmaHyper,log=TRUE),COUNTS[3]))+logJ_34)
    }
    if(COUNTS[4] >= 1){
      iii <- index.i1[[4]]
      lpX <- logit(pX[iii]); lpY <- logit(pY[iii])
      logJ_34 <- -(log(pX[iii])+log(pY[iii])+log(1-pX[iii])+log(1-pY[iii]))
      p[iii] <- rowSums(as.matrix(C[iii,] + 
                                    matrix(dmvnorm(cbind(c(lpX),c(lpY)),
                                                   mean=HMM$muHypo,sigma=HMM$sigmaHypo,log=TRUE),COUNTS[4]))+logJ_34)
    }
    return(p)
  }
  
  g <- function(pX1,pY1,pX2,pY2){
    # return(rowSums(as.matrix(log(dtruncnorm(pX1,0,1,pX2,adspXi))+
    #                            log(dtruncnorm(pY1,0,1,pY2,adspYi)))))
    C1 = log(pnorm(1,pX2,adspXi)-pnorm(0,pX2,adspXi))
    C2 = log(pnorm(1,pY2,adspYi)-pnorm(0,pY2,adspYi))
    return( rowSums(as.matrix(dnorm(pX1,pX2,adspXi,log=TRUE)+dnorm(pY1,pY2,adspYi,log=TRUE)-C1-C2)) )
  }
  accept <- rep(-Inf,N)
  u <- log(runif(N))
  accept <- f(pX.s,pY.s)-f(pX.i,pY.i)+g(pX.i,pY.i,pX.s,pY.s)-g(pX.s,pY.s,pX.i,pY.i)
  pX.i[accept>u] <- pX.s[accept>u]
  pY.i[accept>u] <- pY.s[accept>u]
  accept <- mean(pX.i==pX.s)
  
  return(list(pX.i = pX.i, pY.i = pY.i, 
              index.i1 = index.i1, COUNTS = COUNTS,accept = accept))
}

rtn = function(n,a,b,mu,sd){
  u = runif(n)
  return( qnorm(pnorm((a-mu)/sd)+u*(pnorm((b-mu)/sd)-pnorm((a-mu)/sd)))*sd+mu ) 
}
