

#' Gibbs Sampler to estimate model parameters
#'
#' Given the paired WGBS data, this function calls Sampler for 
#' estimationg the parameters of non-homogeneous hidden Markov model. 
#'
#' @param data  Integrated data, including all the information from WGBS data used in sampler.
#' @param e0 length-4 vector representing the means of priors of \eqn{e_k}. The default value is (5,5,5,5).
#' @param mu01 expectation of \eqn{mu_{X1}}'s  prior. 
#' Default is 0.7.
#' @param mu02 expectation of \eqn{mu_{X2}}'s prior. Default is 0.3
#' @param sigma01 standard deviation of \eqn{mu_{X1}}'s prior. Default is 1.
#' @param sigma02 standard deviation of \eqn{mu_{X2}}'s prior. Default is 1.
#' @param mu03  vector of means of \eqn{mu_3}'s prior. Default is \eqn{(-0.3,0.7)}.
#' @param mu04  vector of means of \eqn{mu_4}'s prior. Default is \eqn{(0.7,-0.3)}
#' @param sigma03 Covariance matrix of \eqn{mu_3}'s prior. Default is \eqn{diag(2)}
#' @param sigma04 Covariance matrix of \eqn{mu_4}'s prior. Default is \eqn{diag(2)}
#' @param vx012 shape parameter of \eqn{sigma_{Xk}}'s prior, k=1,2. Default is 2.
#' @param Sx012 scale parameter of \eqn{sigma_{Xk}}'s prior, k=1,2. Default is 0.1.
#' @param vy012 shape parameter of \eqn{sigma_{Yk}}'s prior, k=1,2. Default is 2.
#' @param Sy012 scale parameter of \eqn{sigma_{Yk}}'s prior, k=1,2. Default is 0.01
#' @param ending ending of each chain. Default is NULL, 
#' and a new chain will be initialized when
#' distance larger than 10,000bp.
#' @param init  list of initial values
#' @param burnin number of iterations for burn-in (including parameter adjustment). Default is 8000.
#' @param iter total number of iterations. Defaut is 10000
#' @param seed a single value, interpreted as an integer. Default is 912.
#' @param ads standard deviation of \eqn{mu_{Xk}}'s proposal,\eqn{k=1,2}
#' @param sstemp standard deviation of \eqn{rho_{Xk}}'s proposal, \eqn{k=3,4}. default is \eqn{(0.05,0.05)}
#' @param adspXi standard deviation of \eqn{p_{X}}'s proposal. default is 0.5
#' @param adspYi standard deviation of \eqn{p_{Y}}'s proposal. default is 0.5
#' @param sshapeBH length-2 vector representing the shape parameter 
#' of \eqn{sigma_{X1}} and \eqn{sigma_{Y1}}'s proposal. Defaut is \eqn{(20,20)}.
#' @param sshapeBL length-2 vector representing theshape parameter 
#' of \eqn{sigma_{X2}} and \eqn{sigma_{Y2}}'s proposal. Defaut is \eqn{(10,10)}.
#' @param shape11 length-2 vector representing the shape parameter 
#' of \eqn{sigma_{13}} and \eqn{sigma_{14}}'s proposal. Defaut is \eqn{(10,10)}.
#' @param shape22 length-2 vector representing the shape parameter 
#' of \eqn{sigma_{23}} and \eqn{sigma_{24}}'s proposal. Defaut is \eqn{(10,10)}.
#' @param adjpar length-2 vector used to adjust parameters in proposals, 
#' the first element smaller 1 and the last element bigger than 1.
#' Default is \eqn{(0.9,1.1)}
#' @param D tunning parameter in model, 
#' which represent the minimun difference between the mean of \eqn{pX_{ij}} and \eqn{pY_{ij}} in the DMR regions.
#' Default is 0.3
#' @param filename output file name
#' @param adjust logical; if TRUE(default), adjust parameters in proposal automatically. 
#' @param nadjust number of iterations for parameter adjustment. Only works when adjust = TRUE. Default is 2000.
#' @param alpha tunning parameter in model, 
#' which used to truncating the covariant matrices of bivariate normals of DMC's responses. 
#' Default is 0.8
#' 
#'
#' @return A list containing
#' \item{pars}{MAP estimations of the parameters in NHMM model}
#' \item{states}{MAP estimations of the status of each CpG site.}
#' \item{pX}{MAP estimations of the underlying methylation level
#' \eqn{p_{X_{ij}} of each CpG site from control groups.}
#' \item{pY}{MAP estimations of the underlying methylation level
#' \eqn{p_{Y_{ij}} of each CpG site from case groups.}
#' In addition, the function will save the parameters, 
#'  posterior probability in each iteration after the burnin in a R Workspace.
#' @export
#'
#' @examples
Sampler <- function(data, e0 = rep(5,4), mu01 = 0.7, mu02=0.3,
                    sigma01 = 1, sigma02 = 1,
                    mu03 = c(-0.3,0.7),mu04 = c(0.7,-0.3),       
                    sigma03 = diag(2), sigma04 = diag(2),
                    vx012 = 2, Sx012 = 0.1, vy012 = 2, Sy012 = 0.01,
                    ending=NULL,init, burnin = 8000, iter = 10000,
                    seed = 0912, alpha = 0.8,ads = rep(0.1,2), 
                    sstemp = rep(0.05,2),  adspXi = 0.5, adspYi = 0.5,
                    sshapeBH = c(20,20), sshapeBL = c(10,10), shape11 = c(10,10),shape22 = c(10,10),
                    adjpar = c(0.9,1.1),
                    D = 0.3,filename,adjust =TRUE,nadjust = 2000){


  
   set.seed(seed)

  pN <- data$X/data$nX
  pT <- data$Y/data$nY
  N <- nrow(data$nX)
  J <- ncol(data$nX)
  L <- data$L
  
  con <- file(paste(filename,"output.log",sep="_"))
  sink(con, append = TRUE)
  sink(con,append=TRUE,type="message")
  print(c("head(L): ",head(L)),quote=FALSE)
  print(c("sum(L>5000 | L<0): ",sum(L>5000 | L<0)),quote=FALSE)
  print(c("min(L): ",min(L)),quote=FALSE)
  print(c("max(L): ",max(L)),quote=FALSE)
  
  
  #I Variables
  HMMStates <- c("Both High", "Both Low", "HyperM", "HypoM")
  nStates <- length(HMMStates)
  StartProb <- array(0,c(nStates,iter+1)) 
  dimnames(StartProb) <- list(state=HMMStates, i=0:iter)
  mu.BH = mu.BL = array(0,iter+1)
  mu.Hyper = mu.Hypo = array(0,c(2,iter+1))
  c.BHX = c.BLX = c.BHY = c.BLY = array(0,iter+1)
  sigma.Hyper = sigma.Hypo = array(0,c(2,2,iter+1))
  e <- array(0,c(nStates,iter+1)) 
  dimnames(e) <- list(state=HMMStates, i=0:iter)
  p.pX = p.pY = p.state = rep(0,N)  #estimation   
  states <- array(0,c(N,iter-burnin))
  dimnames(states) <- list(t=1:N, i=(burnin+1):iter)
  bw <- array(NA, c(nStates, N))
  dimnames(bw) <- list(states = HMMStates, index = 1:N)
  p <- c(); pp <- c(); map=-Inf
  ppp1 = ppp2 = matrix(0,N,4)
  time.record = matrix(0,iter,7)
  accept.i <- array(0,c(15,iter))
  accuracy.i <- c()
  
  #II Initialize a HMM and Variables
  StartProb[,1] <- init$StartProb
  e[,1] <- init$e
  pX.i <- runif(N,0,1); pY.i <- runif(N,0,1)
  pX.i[pX.i<0.01] <- 0.01; pY.i[pY.i<0.01] <- 0.01
  pX.i[pX.i>0.99] <- 0.99; pY.i[pY.i>0.99] <- 0.99
  mu.BH[1] <- init$muBH; mu.BL[1] <- init$muBL;
  mu.Hyper[,1] <- init$muHyper; mu.Hypo[,1] <- init$muHypo;
  c.BHX[1] <- init$cBHX; c.BLX[1] <- init$cBLX
  c.BHY[1] <- init$cBHY; c.BLY[1] <- init$cBLY
  sigma.Hyper[,,1] <- init$sigmaHyper
  sigma.Hypo[,,1] <- init$sigmaHypo
  

  #III Sampling
  
  print('init:')
  print(init)
 
  for ( i in 1:iter){
    
    if ( i %% 100 == 0 ) print(c('iter=',i))
    tempHMM <- list(Iteration=i, States=HMMStates, 
                   StartProb=StartProb[,i], e = e[,i], 
                   muBH=mu.BH[i], muBL=mu.BL[i], 
                   muHyper=mu.Hyper[,i], muHypo=mu.Hypo[,i],
                   cBHX=c.BHX[i], cBLX=c.BLX[i], 
                   cBHY=c.BHY[i], cBLY=c.BLY[i],
                   sigmaHyper=sigma.Hyper[,,i], sigmaHypo=sigma.Hypo[,,i])
    names(tempHMM$e) = HMMStates
    
    time <- proc.time()
    # cat('sample_states,')
    # sample states
    #L <- diff(data$site)
    if(is.null(ending)){
      ending <- c(which(L>10000 | L<0),N)
      starting <- c(1,ending[-length(ending)]+1) 
     
    }else{
      starting <- c(1,ending[-length(ending)]+1)
     }
   
    emp <- emp_all(data,tempHMM, pX.i, pY.i)
    colnames(emp) <- tempHMM$States 
    bw[,ending] <- 0       
    eE <- exp(e[,i])
    bw <- backwardc(tempHMM,t(emp),L,starting,ending)
    dimnames(bw) <- list(states = tempHMM$States, index = 1:N)
    states.ii <- SampleStatesc(tempHMM,t(emp),bw,L,starting,ending)
    states.i <- tempHMM$States[states.ii]
    probObs <- postprobc(tempHMM,t(emp),data$L,states.ii,starting,ending)
    p <- c(p,probObs) 
    C1 <- log( sum(pnorm(c(1,0),mu01,sigma01)*c(1,-1)) )
    C2 <- log( sum(pnorm(c(1,0),mu02,sigma02)*c(1,-1)) )
    prior <-dnorm(mu.BH[i],mu01,sigma01,log=TRUE)-C1 +
      dnorm(mu.BL[i],mu02,sigma02,log=TRUE)-C2 +
      log(dinvgamma(c.BHX[i],vx012,Sx012)) +
      log(dinvgamma(c.BHY[i],vy012,Sy012)) +
      log(dinvgamma(c.BLX[i],vx012,Sx012)) +
      log(dinvgamma(c.BLY[i],vy012,Sy012)) +
      dmvnorm(mu.Hyper[,i],mu03,sigma03,log=TRUE) +
      dmvnorm(mu.Hypo[,i],mu04,sigma03,log=TRUE) +
      dgamma(sqrt(sigma.Hyper[1,1,i]),0.5,0.5)+
      dgamma(sqrt(sigma.Hyper[2,2,i]),0.5,0.5)+
      dgamma(sqrt(sigma.Hypo[1,1,i]),0.5,0.5)+
      dgamma(sqrt(sigma.Hypo[2,2,i]),0.5,0.5)+
      sum(dnorm(e[,i],e0,10,log=TRUE))           #revised
    pp <- c(pp,probObs+prior)      #posterior
    
    # print(c("probObs: ",probObs),quote=FALSE)
    # print(c("pp: ",pp),quote=FALSE)
    
    if(is.na(pp[i])){pp[i]=''} 
    if (i > burnin && pp[i]>map) { 
      map=pp[i]; p.state = states.ii; p.pX=pX.i;p.pY=pY.i; p.HMM = tempHMM
    }    #map estimation
    
    
  
    if(!is.null(data$S)){
      accuracy.i <- c(accuracy.i,sum(states.i==data$S)/N)
    }
    
    time.record[i,1] <- (proc.time() - time)[3]
    time <- proc.time()
    
    if (i > burnin) { states[,i-burnin] <- states.i }
    
    ############adjust sds
    if(adjust){
      if (i <= nadjust ) {
        
          if ( i %% 100 == 0  ) {  
          r.a = apply(accept.i[,((i/100-1)*100+1):(i)],1,sum)
          ads[1] = ifelse(r.a[1]<15,max(ads[1]*adjpar[1],0.001),ads[1])
          ads[2] = ifelse(r.a[2]<15,max(ads[2]*adjpar[1],0.001),ads[2])
          sshapeBH[1] = ifelse(r.a[5]<15,sshapeBH[1]*adjpar[2],sshapeBH[1])
          sshapeBH[2] = ifelse(r.a[6]<15,sshapeBH[2]*adjpar[2],sshapeBH[2])
          sshapeBL[1] = ifelse(r.a[7]<15,sshapeBL[1]*adjpar[2],sshapeBL[1])
          sshapeBL[2] = ifelse(r.a[8]<15,sshapeBL[2]*adjpar[2],sshapeBL[2])
          shape11[1] = ifelse(r.a[9]<15,shape11[1]*adjpar[2],shape11[1])
          sstemp[1] = ifelse(r.a[10]<15,max(sstemp[1]*adjpar[1],0.0001),sstemp[1])
          shape22[1] = ifelse(r.a[11]<15,shape22[1]*adjpar[2],shape22[1])
          shape11[2] = ifelse(r.a[12]<15,shape11[2]*adjpar[2],shape11[2])
          sstemp[2] = ifelse(r.a[13]<15,max(sstemp[2]*adjpar[1],0.0001),sstemp[2])
          shape22[2] = ifelse(r.a[14]<15,shape22[2]*adjpar[2],shape22[2])
          adspXi = ifelse(r.a[15]<15, max(adspXi*adjpar[1],0.001), adspXi)
          adspYi = ifelse(r.a[15]<15, max(adspYi*adjpar[1],0.001), adspYi)
          ads[1] = ifelse(r.a[1]>30,min(ads[1]*adjpar[2],0.1),ads[1])
          ads[2] = ifelse(r.a[2]>30,min(ads[2]*adjpar[2],0.1),ads[2])
          sshapeBH[1] = ifelse(r.a[5]>30,max(sshapeBH[1]*adjpar[1],0.001),sshapeBH[1])
          sshapeBH[2] = ifelse(r.a[6]>30,max(sshapeBH[2]*adjpar[1],0.001),sshapeBH[2])
          sshapeBL[1] = ifelse(r.a[7]>30,max(sshapeBL[1]*adjpar[1],0.001),sshapeBL[1])
          sshapeBL[2] = ifelse(r.a[8]>30,max(sshapeBL[2]*adjpar[1],0.001),sshapeBL[2])
          shape11[1] = ifelse(r.a[9]>30,max(shape11[1]*adjpar[1],0.001),shape11[1])
          sstemp[1] = ifelse(r.a[10]>30,min(sstemp[1]*adjpar[2],0.2),sstemp[1])
          shape22[1] = ifelse(r.a[11]>30,max(shape22[1]*adjpar[1],0.001),shape22[1])
          shape11[2] = ifelse(r.a[12]>30,max(shape11[2]*adjpar[1],0.001),shape11[2])
          sstemp[2] = ifelse(r.a[13]>30,min(sstemp[2]*adjpar[2],0.2),sstemp[2])
          shape22[2] = ifelse(r.a[14]>30,max(shape22[2]*adjpar[1],0.001),shape22[2])
          adspXi = ifelse(r.a[15]>30, min(adspXi*adjpar[2],0.5), adspXi)
          adspYi = ifelse(r.a[15]>30, min(adspYi*adjpar[2],0.5), adspYi)
        }
        
      } 
      
      if (i ==  nadjust+1 ){
        r.a = apply(accept.i[,1:(i-1)],1,sum)
        print('accept:')
        print(r.a/ nadjust)
        print(c("sd of the proposal for pX and pY: ",c(adspXi,adspYi)),quote=FALSE)
        print(c("sd of the proposal for mu.BH and mu.BL: ",ads),quote=FALSE)
        print(c('shape of the proposal for c.BHX and c.BHY:',sshapeBH),quote=FALSE)
        print(c('shape of the proposal for c.BLX and c.BLY:',sshapeBL),quote=FALSE)
        print(c('shape of the proposal for c113 and c223:',shape11[1],shape22[1]),quote=FALSE)
        print(c('shape of the proposal for c114 and c224:',shape11[2],shape22[2]),quote=FALSE)
        print(c("sd of the proposal for sigma_{123} and sigma_{124}: ",sstemp),quote=FALSE) 
      } 

    }else{
      if (i ==  nadjust+1 ){
        print(c("sd of the proposal for pX and pY: ",c(adspXi,adspYi)),quote=FALSE)
        print(c("sd of the proposal for mu.BH and mu.BL: ",ads),quote=FALSE)
        print(c('shape of the proposal for c.BHX and c.BHY:',sshapeBH),quote=FALSE)
        print(c('shape of the proposal for c.BLX and c.BLY:',sshapeBL),quote=FALSE)
        print(c("sd of the proposal for sigma_{123} and sigma_{124}: ",sstemp),quote=FALSE) 
      } 
    }
    
    
    
    
    
    ######################################### sample pX and pY given states 
    # cat('sample_pXpY')
    UpdateP <- SampleUnderProb(tempHMM,data,states.i,
                            pX.i,pY.i,adspXi,adspYi)
    pX.i <- UpdateP$pX.i
    pY.i <- UpdateP$pY.i
    index.i1 <- UpdateP$index.i1
    COUNTS <- UpdateP$COUNTS
    accept.i[15,i] <- UpdateP$accept
    
    ############################################### 
    
    # cat('upadte_startProb,')
    # update start probability
    StartProb[,i+1] <- 
      rdirichlet(1,sapply(tempHMM$States,function(x){sum(states.i[starting]==x)})+1)
    
    # cat('update_TransProb')
    # update transition probability (e_k,k=1,2,3,4)
    posneg <- (states.i[-N] == states.i[-1])         
    posneg[posneg==1] <- 3
    posneg[posneg==0] <- -1
    f <- function(e) {   
      eE <- exp(e)
      trp <- 0.25 + 0.25 * posneg * exp(-L/eE[states.i[-N]])     
      trp[trp<=0] <- 0.001
      trp[ending] <- 1
      sapply(index.i1,function(x){sum(log(trp[x]))})+dnorm(e,e0,10,log=TRUE)
    }
    s <- rnorm(nStates,e[,i],0.5)
    names(s) <- tempHMM$States
    accept <- f(s)-f(e[,i])  #revised
    accept[COUNTS == 0] <- -Inf
    u <- log(runif(nStates))
    e[,i+1] <- e[,i]
    e[accept>u,i+1] <- s[accept>u]
    
    
    
    time.record[i,2] <- (proc.time() - time)[3]
    time <- proc.time()
    
    # cat('update_mu.BH,')
    #update mu.BH  revised
   
    ads0 <- ads[1]           
    #mu.s <- rtn(1,0.5,1,mu.BH[i],ads0)   # 1>mu.s>0.5
	mu.s <- rmtnorm(1,mu.BH[i],ads0,0.5,1)
    f <- function(mu) { 
      C <- log( sum(pnorm(c(1,0),mu,sqrt(c.BHX[i]))*c(1,-1)) )
      C2 <- log(sum(pnorm(c(1,0),mu01,sigma01)*c(1,-1)))
      prior <- dnorm(mu,mu01,sigma01,log=TRUE)-C2
      #prior <- log(dtruncnorm(mu,0,1,mu01,sigma01))   #add
      return( sum(dnorm(pX.i[index.i1$"Both High"],mu,sqrt(c.BHX[i]),log=TRUE)-C)+prior )
    }    
    g <- function(mu1,mu2) {
      #return( log(dtruncnorm(mu1,0.5,1,mu2,ads0)))
      C <- log(sum(pnorm(c(1,0),mu2,ads0)*c(1,-1)))
      return(dnorm(mu1,mu2,ads0,log=TRUE)-C)
    }
    accept <- 0
    if(is.na(COUNTS[1])){COUNTS[1]=''}
    if (COUNTS[1] != 0 ) {
      accept <- min((f(mu.s)-f(mu.BH[i])+g(mu.BH[i],mu.s)-g(mu.s,mu.BH[i])),0)
    }
    u <- log(runif(1,0,1))
    mu.BH[i+1] <- ifelse(accept>u,mu.s,mu.BH[i]) 
   
    
    # cat('update_c.BHX,')
    # update c.BHX   revised
    c.s <- rinvgamma(1,sshapeBH[1],(sshapeBH[1]-1)*c.BHX[i])
    f <- function(x){
      C <- log( sum(pnorm(c(1,0),mu.BH[i+1],sqrt(x))*c(1,-1)) )
      prior <- log(dinvgamma(x,vx012,Sx012))
      return( sum(dnorm(pX.i[index.i1$"Both High"],mu.BH[i+1],sqrt(x),log=TRUE)-C)+prior )
    }
    g <- function(x1,x2){
      return(log(dinvgamma(x1,sshapeBH[1],(sshapeBH[1]-1)*x2)))
    }
    
    accept <- -Inf
    if (COUNTS[1] != 0 ){
      accept <- min((f(c.s)-f(c.BHX[i])+g(c.BHX[i],c.s)-g(c.s,c.BHX[i])),0)
    }
    u <- log(runif(1,0,1))
    c.BHX[i+1] <- ifelse(accept>u,c.s,c.BHX[i])
    
    
    # cat('update_c.BHY,')
    # update c.BHY   revised
    c.s <- rinvgamma(1,sshapeBH[2],(sshapeBH[2]-1)*c.BHY[i])
  
    f <- function(x) { 
      iii <- index.i1$"Both High"
      C <- log(pnorm(1,mean=pX.i[iii],sd=sqrt(x)) - 
                pnorm(0,mean=pX.i[iii],sd=sqrt(x)))
      prior <- log(dinvgamma(x,vy012,Sy012))
      return( sum(dnorm(pY.i[iii],pX.i[iii], sqrt(x),log=TRUE) - C ) + prior)
    }
    g <- function(x1,x2){
      return(log(dinvgamma(x1,sshapeBH[2],(sshapeBH[2]-1)*x2)))
    }
    accept <- -Inf
    if (COUNTS[1] != 0 ) {
      accept <- min((f(c.s)-f(c.BHY[i])+g(c.BHY[i],c.s)-g(c.s,c.BHY[i])),0)
    }
    #if(is.na(accept)){accept <- -Inf}
    u <- log(runif(1,0,1))
    c.BHY[i+1] <- ifelse(accept>u,c.s,c.BHY[i])
    
    
    # cat('update_mu.BL,')
    # update mu.BL revised
    ads0 <- ads[2]
    #mu.s <- rtn(1,0,0.5,mu.BL[i],ads0)   # 0<mu.s<0.5
    mu.s <- rmtnorm(1,mu.BL[i],ads0,0,0.5)
	f <- function(mu) { 
      C <- log( sum(pnorm(c(1,0),mu,sqrt(c.BLX[i]))*c(1,-1)) )
      C2 <- log( sum(pnorm(c(1,0),mu02,sigma02)*c(1,-1)) )
      prior <- dnorm(mu,mu02,sigma02,log=TRUE)-C2
      #prior <- log(dtruncnorm(mu,0,1,mu02,sigma02))   #add
      return( sum(dnorm(pX.i[index.i1$"Both Low"],mu,sqrt(c.BLX[i]),log=TRUE)-C)+prior )
    }
    g <- function(mu1,mu2) {
      #return( log(dtruncnorm(mu1,0,0.5,mu2,ads0)))
      C <- log( sum(pnorm(c(1,0),mu2,ads0)*c(1,-1)) )
      return(dnorm(mu1,mu2,ads0,log=TRUE)-C)
    }
    accept <- 0
    if(is.na(COUNTS[2])){COUNTS[2] = ''}
    if (COUNTS[2] != 0 ) {
      accept <- min((f(mu.s)-f(mu.BL[i])+g(mu.BL[i],mu.s)-g(mu.s,mu.BL[i])),0)
    }
    u <- log(runif(1,0,1))
    mu.BL[i+1] <- ifelse(accept>u,mu.s,mu.BL[i])
   
    
    # cat('update_c.BLX,')
    # update c.BLX revised
    c.s <- rinvgamma(1,sshapeBL[1],(sshapeBL[1]-1)*c.BLX[i])
    f <- function(x){
      C <- log( sum(pnorm(c(1,0),mu.BL[i+1],sqrt(x))*c(1,-1)) )
      prior <- log(dinvgamma(x,vx012,Sx012))
      return( sum(dnorm(pX.i[index.i1$"Both Low"],mu.BL[i+1],sqrt(x),log=TRUE)-C)+prior )
    }
    g <- function(x1,x2){
      return(log(dinvgamma(x1,sshapeBL[1],(sshapeBL[1]-1)*x2)))
    }
    
    accept <- -Inf
    if (COUNTS[2] != 0 ) {
      accept <- min((f(c.s)-f(c.BLX[i])+g(c.BLX[i],c.s)-g(c.s,c.BLX[i])),0)
    }
    u <- log(runif(1,0,1))
    c.BLX[i+1] <- ifelse(accept>u,c.s,c.BLX[i])
    
    
    # cat('update_c.BLY,')
    # update c.BLY   revised
    c.s <- rinvgamma(1,sshapeBL[2],(sshapeBL[2]-1)*c.BLY[i])
    f <- function(x) { 
      iii <- index.i1$"Both Low"
      C <- log(pnorm(1,mean=pX.i[iii],sd=sqrt(x)) - 
                 pnorm(0,mean=pX.i[iii],sd=sqrt(x)))
      prior <- log(dinvgamma(x,vy012,Sy012))
      return( sum(dnorm(pY.i[iii],pX.i[iii], sqrt(x),log=TRUE) - C ) + prior)
    }
    g <- function(x1,x2){
      return(log(dinvgamma(x1,sshapeBL[2],(sshapeBL[2]-1)*x2)))
    }
    accept <- -Inf
    if (COUNTS[1] != 0 ) {
      accept <- min((f(c.s)-f(c.BLY[i])+g(c.BLY[i],c.s)-g(c.s,c.BLY[i])),0)
    }
    u <- log(runif(1,0,1))
    c.BLY[i+1] <- ifelse(accept>u,c.s,c.BLY[i])

    # cat('update_Hyper,')
    #update mu.Hyper 
    
    time.record[i,3] <- (proc.time() - time)[3]
    
    time <- proc.time()
    
    lpX.i <- logit(pX.i); lpY.i <- logit(pY.i)
    iii <- index.i1$"HyperM"
    lpX <- lpX.i[iii];lpY <- lpY.i[iii]
   
     print(COUNTS)
    
     Update_Hyper <-  Update_DMR(COUNTS[3],lpX,lpY,mu.Hyper[,i],sigma.Hyper[,,i],
                                mu03,sigma03,sstemp[1],shape11[1],shape22[1],'HyperM',J,D, alpha)
    mu.Hyper[,i+1] <- Update_Hyper$mu_updated
    
    sigma.Hyper[,,i+1] <- Update_Hyper$sigma_updated
    
    time.record[i,4] <- (proc.time() - time)[4]
    
    time <- proc.time()
    
    # cat('update_Hypo,')
    iii <- index.i1$"HypoM"
    lpX <- lpX.i[iii];lpY <- lpY.i[iii]
    
    
    Update_Hypo <-  Update_DMR(COUNTS[4],lpX,lpY,mu.Hypo[,i],sigma.Hypo[,,i],
                                mu04,sigma04,sstemp[2],shape11[2],shape22[2],'HypoM',J,D, alpha)
    mu.Hypo[,i+1] <-  Update_Hypo$mu_updated
    
    sigma.Hypo[,,i+1] <-  Update_Hypo$sigma_updated
    
    accept.i[1:4,i] = c(mu.BH[i]!=mu.BH[i+1],mu.BL[i]!=mu.BL[i+1],
                        prod(mu.Hyper[,i]!=mu.Hyper[,i+1]),
                        prod(mu.Hypo[,i]!=mu.Hypo[1,i+1]))
    accept.i[5:6,i] = c(c.BHX[i]!=c.BHX[i+1],c.BHY[i]!=c.BHY[i+1])
    accept.i[7:8,i] = c(c.BLX[i]!=c.BLX[i+1],c.BLY[i]!=c.BLY[i+1])
    accept.i[9,i] = (sigma.Hyper[1,1,i]!=sigma.Hyper[1,1,i+1]) 
    accept.i[10,i] = (sigma.Hyper[1,2,i]/sqrt(sigma.Hyper[1,1,i]*sigma.Hyper[2,2,i]) != 
                        sigma.Hyper[1,2,i+1]/sqrt(sigma.Hyper[1,1,i+1]*sigma.Hyper[2,2,i+1]))
    accept.i[11,i] = (sigma.Hyper[2,2,i]!=sigma.Hyper[2,2,i+1])
    accept.i[12,i] = (sigma.Hypo[1,1,i]!=sigma.Hypo[1,1,i+1]) 
    accept.i[13,i] = (sigma.Hypo[1,2,i]/sqrt(sigma.Hypo[1,1,i]*sigma.Hypo[2,2,i]) != 
                                               sigma.Hypo[1,2,i+1]/sqrt(sigma.Hypo[1,1,i+1]*sigma.Hypo[2,2,i+1]))
    accept.i[14,i] = (sigma.Hypo[2,2,i]!=sigma.Hypo[2,2,i+1])
   
    
    time.record[i,7] <- (proc.time() - time)[3]
    time <- proc.time()
    
    
    if ( i %% 500 == 0 ) {
	print(tempHMM) 
	print(COUNTS)
    }

  }
  
  
  #stopCluster(cl)
  colMeans(time.record)
  
  print('\nEstimation_map:')
  print(p.HMM)
  
  pdf(paste(filename,".pdf"))
  par(mfrow=c(5,4),mar=rep(2,4))
  nv = c("muBH","muBL","muHyperX","muHyperY","muHypoX","muHypoY",
         "cBHX","cBHY","cBLX","cBLY",
         "sigmaHyper11","sigmaHyper12","sigmaHyper22",
         "sigmaHypo11","sigmaHypo12","sigmaHypo22"
  )
  plot(mu.BH,type="l",main=nv[1])
  plot(mu.BL,type="l",main=nv[2])
  plot(mu.Hyper[1,],type="l",main=nv[3])
  plot(mu.Hyper[2,],type="l",main=nv[4])
  plot(mu.Hypo[1,],type="l",main=nv[5])
  plot(mu.Hypo[2,],type="l",main=nv[6])
  
  plot(c.BHX,type="l",main=nv[7])
  plot(c.BHY,type="l",main=nv[8])
  plot(c.BLX,type="l",main=nv[9])
  plot(c.BLY,type="l",main=nv[10])
  plot(sigma.Hyper[1,1,],type="l",main=nv[11])
  plot(sigma.Hyper[1,2,],type="l",main=nv[12])
  plot(sigma.Hyper[2,2,],type="l",main=nv[13])
  plot(sigma.Hypo[1,1,],type="l",main=nv[14])
  plot(sigma.Hypo[1,2,],type="l",main=nv[15])
  plot(sigma.Hypo[2,2,],type="l",main=nv[16])
  
  plot(p,type="l",main='p')
  plot(pp,type="l",main='pp')
  plot(tail(pp,iter-nadjust),type="l",main='tail(pp)')
  plot(accuracy.i,type='l',main='accuracy')
  dev.off()
  
  
  
  mu.BH = mu.BH[(burnin+2):(iter+1)]
  mu.BL = mu.BL[(burnin+2):(iter+1)]
  mu.Hyper = mu.Hyper[,(burnin+2):(iter+1)]
  mu.Hypo = mu.Hypo[,(burnin+2):(iter+1)]
  c.BHX = c.BHX[(burnin+2):(iter+1)]
  c.BHY = c.BHY[(burnin+2):(iter+1)]
  c.BLX = c.BLX[(burnin+2):(iter+1)]
  c.BLY = c.BLY[(burnin+2):(iter+1)]
  sigma.Hyper = sigma.Hyper[,,(burnin+2):(iter+1)]
  sigma.Hypo = sigma.Hypo[,,(burnin+2):(iter+1)]
  ee = e[,(burnin+2):(iter+1)]
  SP = StartProb[,(burnin+2):(iter+1)]
  pp = pp[(burnin+2):(iter+1)]
  accuracy.i = accuracy.i[(burnin+2):(iter+1)]
  save(init,mu.BH,mu.BL,mu.Hyper,mu.Hypo,
       c.BHX,c.BHY,c.BLX,c.BLY,
       sigma.Hyper,sigma.Hypo,states,
       ee,SP,p.HMM,p.state,p.pX,p.pY,p,pp,accuracy.i,file=paste(filename,".RData",sep=""))
  
  
  
  map_index <- which.max(pp)-burnin
  print(c("inter of map: ",map_index),quote=FALSE)
  accept_all <- apply(accept.i[,(burnin+1):iter],1,sum)/(iter-burnin)
  print('accept_all')
  print(accept_all)
  sink()
  sink(type="message")
  
  
  ccc = rep(0,N)
  ccc[p.state==1] = 2
  ccc[p.state==2] = 3
  ccc[p.state==3] = 4
  ccc[p.state==4] = 5
  
  jpeg(paste(filename,"HMM.jpeg",sep="_"),width=1436,height=650,quality=100)
  par(mfrow=c(1,2))
  if ( N > 50000 ) kkk = sample(1:N,50000)
  if ( N <= 50000 ) kkk = 1:N
  plot(p.pX,p.pY,col=ccc,pch=4)              #pX and  pY based on map
  lines(c(0,1),c(0,1))                       
  #lines(x,yHyper1);lines(x,yHyper2)          #?? where is the line in the pic
  #lines(x,yHypo1);lines(x,yHypo2)
  plot(data$X/data$nX,data$Y/data$nY,col=ccc,pch=4)  
  lines(c(0,1),c(0,1))
  #lines(x,yHyper1);lines(x,yHyper2)
  #lines(x,yHypo1);lines(x,yHypo2)
  dev.off()
  
  
  pdf(paste(filename,"_con",".pdf"))
  par(mfrow=c(5,4),mar=rep(2,4))
  nv = c("muBH","muBL","muHyperX","muHyperY","muHypoX","muHypoY",
         "cBHX","cBHY","cBLX","cBLY",
         "sigmaHyper11","sigmaHyper12","sigmaHyper22",
         "sigmaHypo11","sigmaHypo12","sigmaHypo22"
  )
  plot(mu.BH,type="l",main=nv[1])
  plot(mu.BL,type="l",main=nv[2])
  plot(mu.Hyper[1,],type="l",main=nv[3])
  plot(mu.Hyper[2,],type="l",main=nv[4])
  plot(mu.Hypo[1,],type="l",main=nv[5])
  plot(mu.Hypo[2,],type="l",main=nv[6])
  
  plot(c.BHX,type="l",main=nv[7])
  plot(c.BHY,type="l",main=nv[8])
  plot(c.BLX,type="l",main=nv[9])
  plot(c.BLY,type="l",main=nv[10])
  plot(sigma.Hyper[1,1,],type="l",main=nv[11])
  plot(sigma.Hyper[1,2,],type="l",main=nv[12])
  plot(sigma.Hyper[2,2,],type="l",main=nv[13])
  plot(sigma.Hypo[1,1,],type="l",main=nv[14])
  plot(sigma.Hypo[1,2,],type="l",main=nv[15])
  plot(sigma.Hypo[2,2,],type="l",main=nv[16])
  
  plot(p[(burnin+1):(iter-1)],type="l",main='p')
  plot(pp,type="l",main='pp')
  plot(accuracy.i,type='l',main='accuracy')
  dev.off()
  
 
  return(list(pars = p.HMM, states = p.state,
              pX = p.pX, pY = p.pY
              ))
 
}


