#' Sample state and underlying prob given the parameter in the NHMM model
#'
#' @param data   Integrated data, including all the information from WGBS data used in sampler .
#' @param ending ending of each chain. Default is NULL, 
#' and a new chain will be initialized when
#' distance larger than 10,000bp.
#' @param burnin  number of iterations for burn-in (including parameter adjustment). Default is 8000.
#' @param iter total number of iterations. Defaut is 10000
#' @param seed a single value, interpreted as an integer. Default is 912.
#' @param filename output file name
#' @param init list of initial values
#' @param pX.i0 vector representing the initial values of p_{X_{ij}}, default is NULL.
#' @param pY.i0 vector representing the initial values of p_{Y_{ij}}, default is NULL.
#' @param nadjust number of iterations for parameter adjustment. Only works when adjust = TRUE. Default is 2000.
#' @param adjust logical; if TRUE(default), adjust parameters in proposal automatically. 
#'
#' @return A list containing
#' \item{DMR.prob}{ probability of being DMR for each CpG site. }
#' \item{states}{MAP estimations of the status of each CpG site. 
#' 1 to 4 represent 'Both High','Both Low','HyperM' and 'HypoM' respectively. }
#' \item{DMCs}{MAP estimations of the status of each CpG site. 
#' 0 and 1 represent 'nonDMR' and 'DMR' respectively. }
#' @export
#'
#' @examples
#' 



call_DMCs <- function(data,ending=NULL,init, burnin = 80, iter = 100,
                      seed = 0912,pX.i0=NULL,pY.i0=NULL,
                      nadjust=2000, filename,adjust =TRUE){
  
  set.seed(seed)
  L <- diff(data$site)
  pN <- data$X/data$nX
  pT <- data$Y/data$nY
  N <- nrow(data$nX)
  J <- ncol(data$nX)
  
  con <- file(paste(filename,"output.log",sep="_"))
  sink(con, append = TRUE)
  sink(con,append=TRUE,type="message")
  print(c("head(L): ",head(L)),quote=FALSE)
  print(c("sum(L>5000 | L<0): ",sum(L>5000 | L<0)),quote=FALSE)
  print(c("min(L): ",min(L)),quote=FALSE)
  print(c("max(L): ",max(L)),quote=FALSE)
  print(c('N:',N))
  
  if(is.null(ending)){
    ending <- c(which(L>10000 | L<0),N)
    starting <- c(1,ending[-length(ending)]+1) 
    
  }else{
    starting <- c(1,ending[-length(ending)]+1)
  }
  
  
  #I Variables
  HMMStates <- c("Both High", "Both Low", "HyperM", "HypoM")
  nStates <- length(HMMStates)
  StartProb <- array(0,c(nStates,iter+1))
  p.pX = p.pY = p.state = rep(0,N)  #estimation   
  states <- array(0,c(N,iter-burnin))
  dimnames(states) <- list(t=1:N, i=(burnin+1):iter)
  bw <- array(NA, c(nStates, N))
  dimnames(bw) <- list(states = HMMStates, index = 1:N)
  p <- c(); pp <- c(); map=-Inf
  ppp1 = ppp2 = matrix(0,N,4)
  accuracy.i <- c()
  auc <- c()
  accept.i <- numeric()
  
  #II Initialize a HMM and Variables
  StartProb[,1] <- init$StartProb
  if(is.null(pX.i0)){
     pX.i <- runif(N,0,1)
     pX.i[pX.i<0.01] <- 0.01
      pX.i[pX.i>0.99] <- 0.99
     
  }else{pX.i <- pX.i0}
  if(is.null(pY.i0)){
     pY.i <- runif(N,0,1)

    pY.i[pY.i<0.01] <- 0.01
    pY.i[pY.i>0.99] <- 0.99
  }else{pY.i <- pY.i0}
 
  # pX.i <- init$p.pX
  # pY.i <- init$p.pY
  
  adspXi = 0.1; adspYi = 0.1  #sd of the proposal density of pX,pY
  
  #III Sampling
  
  print('init:')
  print(init)
  
  for ( i in 1:iter){
    
    if ( i %% 100 == 0 ) print(c('iter=',i))
    tempHMM <- list(Iteration = i, States = HMMStates, 
                    StartProb = init$StartProb, e = init$e, 
                    muBH = init$muBH, muBL = init$muBL, 
                    muHyper = init$muHyper, muHypo = init$muHypo,
                    cBHX = init$cBHX, cBLX = init$cBLX, 
                    cBHY = init$cBHY, cBLY = init$cBHY,
                    sigmaHyper = init$sigmaHyper, sigmaHypo = init$sigmaHypo)
    names(tempHMM$e) = HMMStates
    
    emp <- emp_all(data,tempHMM, pX.i, pY.i)
    colnames(emp) <- tempHMM$States 
    bw[,ending] <- 0 
    e = init$e
    eE <- exp(e)
    bw <- backwardc(tempHMM,t(emp),L,starting,ending)
    dimnames(bw) <- list(states = tempHMM$States, index = 1:N)
    states.ii <- SampleStatesc(tempHMM,t(emp),bw,L,starting,ending)
    states.i <- tempHMM$States[states.ii]
    
    probObs <- postprobc(tempHMM,t(emp),data$L,states.ii,starting,ending)
    p <- c(p,probObs)  
    
    pp <- c(pp,probObs)      #posterior
    
    # print(c("probObs: ",probObs),quote=FALSE)
    # print(c("pp: ",pp),quote=FALSE)
    
    if (i > burnin && pp[i]>map) {
      map=pp[i]; p.state = states.ii; p.pX=pX.i;p.pY=pY.i; p.HMM = tempHMM
      states.prob <- callDMRc(tempHMM,t(emp),bw,L,starting,ending)
      DMR.prob <- 1-states.prob[2,]
      states_MAP <- states.i
    }    #map estimation
    
    
    
    if(!is.null(data$S)){
      accuracy.i <- c(accuracy.i,sum(states.i==data$S)/N)
    }
    
    # time.record[i,1] <- (proc.time() - time)[3]
    # time <- proc.time()
    
    if (i > burnin) { states[,i-burnin] <- states.i }
    
    if(adjust){
      if (i <= nadjust ) {
        if ( i %% 10 == 0 && i > 51 ) {
          r.a = sum(accept.i[(i-51):(i-1)])
          adspXi = ifelse(r.a<25, max(adspXi*0.95,0.001), adspXi)
          adspYi = ifelse(r.a<25, max(adspYi*0.95,0.001), adspYi)
          adspXi = ifelse(r.a>35, min(adspXi*1.05,0.1), adspXi)
          adspYi = ifelse(r.a>35, min(adspYi*1.05,0.1), adspYi)
        }
        
      }
    }
    
    
    ######################################### sample pX and pY given states 
    # cat('sample_pXpY')
    UpdateP <- SampleUnderProb(tempHMM,data,states.i,
                               pX.i,pY.i,adspXi,adspYi)
    pX.i <- UpdateP$pX.i
    pY.i <- UpdateP$pY.i
    index.i1 <- UpdateP$index.i1
    COUNTS <- UpdateP$COUNTS
    accept.i[i] <- UpdateP$accept
    
    if ( i %% 100 == 0 ) print(tempHMM)  
    
    
  }
  
  
  #stopCluster(cl)
  # colMeans(time.record)
  
  print('\nEstimation_map:')
  print(p.HMM)
  
  pdf(paste(filename,".pdf",sep=''))
  
  plot(p,type="l",main='p')
  plot(pp,type="l",main='pp')
  # plot(accuracy.i,type='l',main='accuracy')
  dev.off()
  
  pp = pp[(burnin+2):(iter+1)]
  
  if(!is.null(data$S)){
    accuracy.i = accuracy.i[(burnin+2):(iter+1)]
    save(init,states,p.state,states_MAP,
         p.pX,p.pY,p,pp,accuracy.i,file=paste(filename,".RData",sep=""))
  }else{
     save(init,states,p.state,states_MAP,
          p.pX,p.pY,p,pp,file=paste(filename,".RData",sep=""))
  }
  
  map_index <- which.max(pp)-burnin
  print(c("inter of map: ",map_index),quote=FALSE)
  
  sink()
  sink(type="message")
  
  
  
  ccc = rep(0,N)
  ccc[p.state==1] = 2
  ccc[p.state==2] = 3
  ccc[p.state==3] = 4
  ccc[p.state==4] = 5
  
  jpeg(paste(filename,"HMM.jpeg",sep="_"),width=1436,height=650,quality=100)
  par(mfrow=c(1,2))
  if ( N > 50000 ) kkk = sample(1:N,50000)
  if ( N <= 50000 ) kkk = 1:N
  plot(p.pX,p.pY,col=ccc,pch=4)              #pX and  pY based on map
  lines(c(0,1),c(0,1))                       
  plot(data$X/data$nX,data$Y/data$nY,col=ccc,pch=4)  
  lines(c(0,1),c(0,1))
  # plot(auc,type='l',col=ccc,pch=4)  
  dev.off()
  
  DMCs <- p.state
  DMCs[DMCs==1] <- 0
  DMCs[DMCs==2] <- 0
  DMCs[DMCs==3] <- 1
  DMCs[DMCs==4] <- -1
  
  return(list(DMR.prob = DMR.prob, STATES=p.state, DMCs = DMCs))
  
}



#####

