
#' Update parameters of DMCs
#' 
#'Updata the mean \eqn{mu_k} and covariance matrix \eqn{sigma_{Xk}}
#'of methylation levels of DMCs('HyperM' or 'HypoM) for \eqn{k=3,4}
#'
#' @param NinDMR number of CpG sites in 'Hyper'('Hypo') status
#' @param lpX  a numeric vector of the logit-transformed methylation levels of 
#' CpG sites in 'Hyper'('Hypo') status from the control groups.  
#' @param lpY a numeric vector of the logit-transformed methylation levels of 
#' CpG sites in 'Hyper'('Hypo') status from the case groups.
#' @param mu mean from the previous iteration.
#' @param sigma covariance matrix from the previous iteration.
#' @param mu0  vector of means of \eqn{mu_k}'s prior for \eqn{k=3,4}. 
#' @param sigma0 Covariance matrix of \eqn{mu_k}'s prior
#' @param sstemp standard deviation of \eqn{rho_{Xk}}'s proposal
#' @param shape11 the shape parameter 
#' of \eqn{sigma_{1k}}'s proposal for \eqn{k=3,4}.
#' @param shape22 the shape parameter 
#' of \eqn{sigma_{2k}}'s proposal for \eqn{k=3,4}.
#' @param State 'HyperM' or 'HypoM'
#' @param J number of pairs
#' @param D tunning parameter in model, 
#' which represent the minimun difference between the mean of \eqn{pX_{ij}} and \eqn{pY_{ij}} in the DMR regions.
#' @param alpha tunning parameter in model, 
#' which used to truncating the covariant matrices of bivariate normals of DMC's responses. 
#'
#' @return A list containing
#' \item{mu_updated}{Updated mean of the logit-transformed methylation levels}
#' \item{sigma_updated}{Updated covariance matrixof the logit-transformed methylation levels}
#' @export
#'
#' @examples
Update_DMR <- function(NinDMR,lpX,lpY,mu,sigma,mu0,sigma0,sstemp,shape11,shape22,State,J,D, alpha){
  
  mu_updated <- rep(0,2)
  sigma_updated <- matrix(rep(0,4),2)
  if(NinDMR!=0){
    # cat('[1],')
    #mu.Hyper[1] or mu.Hypo[1]
    meanX <- c(mean(lpX),mean(lpY))
    sigma01 <- sigma0[1,1]
    mu01 <- mu0[1]
    bound1 <- Bound_mu1(State,mu,sigma,alpha,D)
    
    s11 <- sigma[1,1]
    s12 <- sigma[1,2]
    s22 <- sigma[2,2]
    detS <- s11*s22-s12^2
    
    newmean <- (sigma01* NinDMR*J*(s22*meanX[1]-s12*meanX[2]+s12*mu[2]) +
                  detS*mu01 ) / (sigma01*s22*NinDMR*J+detS)
    
    newsigma <- detS*sigma01/(sigma01*s22*NinDMR*J+detS)
    
    if(State=='HyperM'){
      s1= rtruncnorm(1,-15,bound1,newmean,sqrt(newsigma))
      mu_updated[1] = s1 
      if(is.na(s1)){mu_updated[1]=bound1}
    }else{
      s1 = rtruncnorm(1,bound1,15,newmean,sqrt(newsigma))
      mu_updated[1] = s1
      if(is.na(s1)){mu_updated[1]=bound1}
    }
    
    # cat('[2],')
    #mu.Hyper[2] or mu.Hypo[2]
    sigma02 <- sigma0[2,2]
    mu02 <- mu0[2]
    bound2 <- Bound_mu2(State,mu_updated,sigma,alpha,D)
    
    newmean <- (sigma02*NinDMR*J*(s11*meanX[2]-s12*meanX[1]+s12*mu_updated[1]) +
                  detS*mu02 ) / (sigma02*s11*NinDMR*J+detS)
    newsigma <- detS*sigma02/(sigma02*s11*NinDMR*J+detS)
    
    if(State == 'HyperM'){
      s2 = rtruncnorm(1,bound2,15,newmean,sqrt(newsigma))
      mu_updated[2] = s2
      if (is.na(s2)){mu_updated[2]=bound2}
    }else{
      mu_updated[2] = s2 = rtruncnorm(1,-15,bound2,newmean,sqrt(newsigma))
      if (is.na(s2)){mu_updated[2]=bound2}
    }
    
  }else{mu_updated = mu}
  
  ##print(c('mu=',mu_updated))
  
  W1  <- sum((lpX-mu_updated[1])^2)
  W2 <- sum((lpY-mu_updated[2])^2)
  W12 <- sum((lpX-mu_updated[1])*(lpY-mu_updated[2]))
  
  if (NinDMR > 1){
    
    # cat('[12],')
    # propose a new sigma_{12} from a truncnorm distribution.
    
    ##print(sigma)
    
    s11 <- sigma[1,1]
    s22 <- sigma[2,2]
    s12 <- sigma[1,2]
    rho <- s12/sqrt(s11*s22)
    R12 <- Bound_rho(mu_updated,sigma,alpha)
    if(is.na(R12)){R12 <- ''}
    if(R12>1){
      return(Update_DMR(NinDMR,lpX,lpY,mu,sigma,mu0,sigma0,sstemp,v0,S0,State,J,D, alpha))
    }else{
      rho.s <- rtruncnorm(1, R12 , 1, mean = rho, sd = sstemp)
      if(is.na(rho.s)){
        rho.s <- (rho+1)/2
      }
      
      
      f <- function(x){
        res <- -NinDMR/2*log(1-x^2)-
          1/(2*(1-x^2))*(W1/s11+W2/s22-2*x*W12/sqrt(s11*s22))
        return(res)
      }
      
      
      # Sigma.s <- matrix(c(s11,sigma.s,sigma.s,s22),2)
      
      accept <- min((f(rho.s)-f(rho))+
                      (dnorm(rho,mean=rho.s,sd=sstemp,log=TRUE)-
                         dnorm(rho.s,mean=rho,sd=sstemp,log=TRUE)),0)
      if(is.nan(accept)){accept <- 0}
      # #print(c("rho","rho.s","accept","u"))
      # #print(c(rho,rho.s,accept,u))
      # #print(c(f(rho.s),f(rho),log(dtruncnorm(rho,R12,1, mean=rho.s,sd=sstemp)),log(dtruncnorm(rho.s,R12, 1, mean=rho,sd=sstemp))))
      
      u = log(runif(1,0,1))
      
      sigma_updated[1,2] <- ifelse(accept>u,rho.s*sqrt(s11*s22),sigma[1,2])
      sigma_updated[2,1] <- sigma_updated[1,2]
      
      # #print(c('rho=',rho.s,sigma_updated[1,2]/sqrt(s11*s22)))
      
    }
    
    
    
    # cat('[11],')
    
    s12 <- sigma_updated[1,2]
    s22 <- sigma[2,2]
    s11 <- sigma[1,1]
    rho <- s12/sqrt(s11*s22)
    Sigma <- matrix(c(s11,s12,s12,s22),2)
    R11 <- Bound_sigma1(mu_updated,Sigma,alpha)
    
    # sigma1.s <- 1/rtgamma(1,lower = 1/R11[2], upper = 1/R11[1],
    #                       shape11,(shape11-1)*sqrt(s11))
    
    sigma1.s <- rtgamma(1,lower=R11[1],upper=R11[2],shape11,sqrt(s11)/shape11)
    
    f <- function(x){
      res <- -NinDMR*log(x)-
        1/(2*(1-rho^2))*(W1/x^2+W2/s22-2*rho*W12/(x*sqrt(s22)))
      # prior <- log(dinvgamma(x,10,(10-1)*0.1))
      prior <- log(dgamma(x,0.5,0.5))
      return(res+prior)
    }
    
    g <- function(x,y){
      
      # return(log(dinvgamma(x,shape11,(shape11-1)*y)))
      return(log(dgamma(x,shape11,scale=y/shape11)))
    }
    
    accept <- min((f(sigma1.s)-f(sqrt(s11)))+
                    (g(sqrt(s11),sigma1.s)-
                       g(sigma1.s,sqrt(s11))),0)
    
    if(is.nan(accept)){accept <- -Inf}
    #print(c("f(sigma1.s)","f(sqrt(s11))","g(sqrt(s11),sigma1.s)","g(sigma1.s,sqrt(s11))"))
    #print(c(f(sigma1.s),f(sqrt(s11)),g(sqrt(s11),sigma1.s),g(sigma1.s,sqrt(s11))))
    u = log(runif(1,0,1))
    
    if(accept>u){
      sigma_updated <- matrix(c(sigma1.s^2,rho*sigma1.s*sqrt(s22),
                                rho*sigma1.s*sqrt(s22),s22),2)
    }else{
      sigma_updated <- Sigma
    }
    # sigma_updated[1,1] <- ifelse(accept>u,sigma1.s^2,sigma[1,1])
    
    ##print(c('s11=',sigma_updated[1,1]))
    #print(sigma_updated)
    #print(c('rho11=',sigma_updated[1,2]/sqrt(sigma_updated[1,1]*s22)))
    
    # cat('[22],')
    s11 <- sigma_updated[1,1]
    s12 <- sigma_updated[1,2]
    s22 <- sigma_updated[2,2]
    rho <- s12/sqrt(s11*s22)
    Sigma <- matrix(c(s11,s12,s12,s22),2)
    R22 <- Bound_sigma2(mu_updated,Sigma,alpha)
    
    
    # sigma2.s <- 1/rtgamma(1,lower = 1/R22[2], upper = 1/R22[1],
                          # shape22,(shape22-1)*sqrt(s22))
    sigma2.s <- rtgamma(1,lower = R22[1], upper = R22[2],
                       shape22, sqrt(s22)/shape22)
    
    f <- function(x){
      res <- -NinDMR*log(x)-
        1/(2*(1-rho^2))*(W1/s11+W2/x^2-2*rho*W12/(sqrt(s11)*x))
      # prior <- log(dinvgamma(x,10,(10-1)*0.1))
      prior <- log(dgamma(x,0.5,0.5))
      return(res+prior)
    }
    
    g <- function(x,y){
      
      # return(log(dinvgamma(x,shape22,(shape22-1)*y)))
      return(log(dgamma(x,shape22,scale=y/shape22)))
      
    }
    
    accept <- min((f(sigma2.s)-f(sqrt(s22)))+
                    (g(sqrt(s22),sigma2.s)-
                       g(sigma2.s,sqrt(s22))),0)
    #print(c("f(sigma2.s)","f(sqrt(s22))","g(sqrt(s22),sigma2.s)","g(sigma2.s,sqrt(s22))"))
    #print(c(f(sigma2.s),f(sqrt(s22)),g(sqrt(s22),sigma2.s),g(sigma2.s,sqrt(s22))))
    
    if(is.nan(accept)){accept <- -Inf}
    
    u = log(runif(1,0,1))
    
    if(accept>u){
      sigma_updated <- matrix(c(s11,rho*sqrt(s11)*sigma2.s,
                                rho*sqrt(s11)*sigma2.s,sigma2.s^2),2)
    }else{
      sigma_updated <- Sigma
    }
    # sigma_updated[2,2] <- ifelse(accept>u,sigma2.s^2,sigma[2,2])
    
    #print(c('s22=',sigma2.s,sigma_updated[2,2]))
    #print(sigma_updated)
    #print(c('rho22=',sigma_updated[1,2]/sqrt(sigma_updated[2,2]*s11)))
    
    
  }else{
    sigma_updated <- sigma
  }
  
  return(list(mu_updated = mu_updated,sigma_updated=sigma_updated))
}










